# Meldingsformidler - Common

Et internt bibliotek for delte verktøy i meldingsformidler-applikasjonene.

# Verktøy

### OpenAmClient

En klient for å koble seg til OpenAm for å hente en JWT-token. Den cacher tokenet som hentes,
og henter nytt når tokenet utløper.
Den settes opp slik:

Java:

```java
new OpenAmClient(
        Oauth2Client.Companion.create(openAmUrl, openAmUsername, openAmPassord, oidcClientId, oidcClientSecret), 
        TokenValidator.create()
);
```

I kotlin er `TokenValidator` satt opp som default. `TokenValidator` kan settes om man ønsker
å override dens parametere (se lenger ned).

```kotlin
OpenAmClient(Oauth2Client.create(openAmUrl, openAmUsername, openAmPassord, oidcClientId, oidcClientSecret))
```


| Parameter        | Beskrivelse                             |
|------------------|-----------------------------------------|
| openAmUrl        | URl til openAm access token             |
| openAmUsername   | Brukernavnet for å koble seg til OpenAM |
| openAmPassword   | Passordet for å koble seg til OpenAM    |
| oidcClientId     | Brukernavnet for tjenestebrukeren       |
| oidcClientSecret | Passordet for tjenestebrukeren          |

**TokenValidator har to optional-parametere:**

`timeLeftToBeConsideredExpired` - Tid igjen til tokenet anses som utgått (**default = 30s**)  
`timeLeftToBeConsideredAlmostExpired` - Tid igjen til det prøves å hente ut nytt token (**default = 15m**)

# Secure Text

I noen tilfeller er det nyttig å logge sensitiv informasjon i produksjon. `SecureText` gir deg denne muligheten gjennom å bruke kryptering med RSA. 

Først må du generere et RSA nøkkelpar. Dette kan du gjøre [online](https://cryptotools.net/rsagen) eller via terminalen. Ta vare på både public og private key. **Husk å aldri commit kode med private key!**

Krypter så teksten slik:
```
  val publicKeyRsa = SecureText.getPublicKey(
        """
         -----BEGIN PUBLIC KEY-----
         MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQCNg8U+OkXe687QO2OgEaucV26k
         WqRoARVfF+UA5AuQSInMHC9xOMEHCFeMVrt2NEBHsFZiTl46bJH5voHh0z35cKfO
         OS+BBPiuniC8r0npvoL1e1oiO9AqqPDAuyyGyK22tqWLiCqEWZA/XG7fZhHpv7+N
         lpWBFazwKlqrvDHlWQIDAQAB
         -----END PUBLIC KEY-----
        """.trimIndent())

 val encryptedChunks: String = SecureText.encrypt(
     publicKey = publicKeyRsa,
     plainText = "The text to encrypt - add it to this text block <3",
     chunkSize = 30
 )

println(encryptedChunks)
```
**PS: Public key kan fint ligge i produksjonskoden, den er ikke hemmelig.**

I eksemplet over vil teksten bli kryptert i tekstbolker på 30 tegn. Denne kan være større om nøkkelen er lang. Bare test med større chunk size helt til det krasjer. Jo lengre jo mindre støy i loggene.  


Når dette er prodsatt så kan du hente ut den krypterte teksten fra loggen, f.eks. en som ser ut som denne:

```shell
[YBtRexPla+uwnG9tuDbeid8u2gWa3pK+WLsXjlZJDsDjs+CBwXKvJSe359gd6QBZFYkWsmsJeeU9ULPU6APsgSOxX5+JOUuD9BNu+ZW5pu0FTNk+yfCckSmO79heu2prwnnWvuECzRaHqXoBhXrrjMZyQLMWDxTuQ5J2obmOaYg=, cMBKzrFDbl3pE8vsP9yP+lZO+u8tMleamZMrSO3Mpqh9UzBiudnKE5mxixsHmiDU9jUKC2GydokuhdxAQoDbbUzKcSRB9kt4hIS51TZuakaSxKRd0Vgho5vPkbbXwXqN1NdyPTAcEBErkOYi8joLohbZgov/zG/aHQ36poJRF6k=]
```

Denne strengen kan du sende inn lokalt på din maskin til `SecureText.DecryptToString()`. Lag for eksempel en test som ser slik ut:

```kotlin
@Test
fun `decrypt content`() {
   Security.addProvider(
      BouncyCastleProvider()
   )
   val privateKey: PrivateKey = SecureText.getPrivateKey(
      """
            -----BEGIN RSA PRIVATE KEY-----
            private nøkkel her (IKKE COMMIT DENNE!)
            -----END RSA PRIVATE KEY-----
        """.trimIndent()
   )
   val encryptedChunks =
      "[encrypted chunks here]"
   val content = SecureText.decryptToString(privateKey, encryptedChunks)
   print("Decrypted content: $content")
}
```

Kjør testen og se dekryptert innhold.

# Release av ny versjon

1. Utvikle ny funksjonalitet i branchen din.
1. Få koden godkjent i branchen, ikke merge.
1. Sett versjonsnummer du vil ha og last opp artefakt til Artrepo:
   ```shell
   mvn versions:set -DnewVersion=<X.Y.Z> -DprocessAllModules //F.eks. mvn versions:set -DnewVersion=1.0.0 -DprocessAllModules
   mvn clean deploy
   ```
1. Commit med melding:
   ```shell
   ci: <Beskrivelse av feature>, A.B.C-SNAPSHOT -> X.Y.Z
  
   <JIRA-NUMMER>
   ```
   , f.eks.

   ```shell
   ci: Integrasjon med system, 1.0.0-SNAPSHOT -> 1.1.0
  
   SAMPORT-57
   ```
1. Lag en tag for releaseversjonen: `git tag vX.Y.Z`, f.eks. `git tag v1.1.0`.
1. Push tags: `git push --tags`.
1. Sett versjonen til neste utviklingsversjon:
   ```
   mvn versions:set -DnewVersion=<X.Y.Z+1-SNAPSHOT> -DprocessAllModules //mvn versions:set -DnewVersion=2.0.1-SNAPSHOT -DprocessAllModules
   ```
1. Commit med melding: `ci: X.Y.Z -> X.Y.Z+1-SNAPSHOT`, f.eks. `ci: 1.1.0 -> 1.1.1-SNAPSHOT`.
1. Push.
1. Merge branch til master.


