package no.vegvesen.meldingsformidler.common.domain

abstract class Postadresse(
    open val navn: String,
    open val adresseLinjer: AdresseLinjer,
    open val landNavn: String,
    open val adresseType: AdresseType,
)

@Suppress("unused")
data class PostadresseNorsk(
    override val navn: String,
    override val adresseLinjer: AdresseLinjer,
    override val landNavn: String = "NORWAY",
    override val adresseType: AdresseType,
    val postNummer: String,
    val postSted: String,
) : Postadresse(navn, adresseLinjer, landNavn , adresseType)

@Suppress("unused")
data class PostadresseUtenlandsk(
    override val navn: String,
    override val adresseLinjer: AdresseLinjer,
    override val landNavn: String,
    override val adresseType: AdresseType,
) : Postadresse(navn, adresseLinjer, landNavn , adresseType)

@Suppress("unused")
data class AdresseLinjer(
    val linje1: String,
    val linje2: String = "",
    val linje3: String = "",
    val linje4: String = "",
) {

    init {
        require(linje1.isNotBlank()) { "The first line of an address is mandatory. Received linje1='$linje1', linje2='$linje2', linje3='$linje3' and linje4='$linje4'." }
    }

    fun toLines(): List<String> = listOf(linje1, linje2, linje3, linje4)


    companion object {

        fun from(adresselinje1: String, adresselinje2: String?, adresselinje3: String?, adresselinje4: String?): AdresseLinjer {
            return AdresseLinjer(
                adresselinje1,
                adresselinje2 ?: "",
                adresselinje3 ?: "",
                adresselinje4 ?: "",
            )
        }

        fun from(list: List<String?>): AdresseLinjer {
            val defaultValue = ""
            return AdresseLinjer(
                list.getOrNull(0) ?: defaultValue,
                list.getOrNull(1) ?: defaultValue,
                list.getOrNull(2) ?: defaultValue,
                list.getOrNull(3) ?: defaultValue,
            )
        }

        fun fillEmptyLinjer(list: List<String?>): List<String> =
            list.map { it ?: "" } + List(4 - list.size) { "" }
    }
}
