package no.vegvesen.meldingsformidler.common.domain

import org.apache.commons.lang3.builder.EqualsBuilder

abstract class StringWithLength(val value: String, val length: StringLength) {

    init {
        if (value.length > length.maxLength) {
            throw IllegalArgumentException("${this.javaClass.simpleName} has length ${value.length}, which is greater than max length ${length.maxLength} and is not valid.")
        }
    }

    override fun equals(other: Any?): Boolean {
        if (this === other) return true
        if (other !is StringWithLength) return false
        if (javaClass != other.javaClass) return false

        return EqualsBuilder()
            .append(value, other.value)
            .append(length, other.length)
            .isEquals
    }

    override fun toString(): String {
        return "${this.javaClass.simpleName}:(value='$value' Maxlength=$length)"
    }
}

enum class StringLength(val maxLength: Int) {
    Max100(100),
    Max255(255)
}
