@file:Suppress("unused")

package no.vegvesen.meldingsformidler.common.extensions

import java.time.Instant
import java.time.LocalDate
import java.time.LocalDateTime
import java.time.LocalTime
import java.time.OffsetDateTime
import java.time.ZoneId
import java.time.ZoneOffset
import java.time.format.DateTimeFormatter

object DateTimeExtensions {

    fun LocalDateTime.toInstantAtStartOfDay(): Instant =
        this.toLocalDate().atStartOfDay().toInstant(ZoneOffset.ofHours(0))

    fun LocalDateTime.toInstantAtEndOfDay(): Instant =
        this.toLocalDate().atTime(LocalTime.MAX).toInstant(ZoneOffset.ofHours(0))

    fun LocalDateTime.toInstantAtEuropeParisTimeZone(): Instant =
        this.atZone(ZoneId.of("Europe/Paris")).toInstant()

    fun LocalDate.toInstantAtEuropeParisTimeZone(): Instant =
        this.atStartOfDay().toInstantAtEuropeParisTimeZone()

    fun LocalDateTime.toFormattedStringWithZone(): String {
        val zoneOffset = ZoneId.of("Europe/Oslo").rules.getOffset(this)
        val offsetDateTime = OffsetDateTime.of(this, zoneOffset)
        val formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ssXXX")
        return offsetDateTime.format(formatter)
    }

}
