package no.vegvesen.meldingsformidler.common.openamclient

import io.mockk.every
import io.mockk.mockk
import kotlin.time.Duration.Companion.minutes
import kotlin.time.Duration.Companion.seconds
import org.assertj.core.api.Assertions.assertThat
import org.junit.jupiter.api.BeforeAll
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestInstance

@TestInstance(TestInstance.Lifecycle.PER_CLASS)
class TokenValidatorTest {

    private val validator = TokenValidator()

    private val mockBearerToken: OpenAmClient.BearerToken = mockk(relaxed = true)

    @BeforeAll
    fun beforeAll() {
        every { mockBearerToken.token } returns "somethingsubstantial"
    }

    @Test
    fun `token with more than 15 minutes left is valid`() {
        val expected = TokenValidator.Result.Valid
        every { mockBearerToken.remainingValidTime() } returns 16.minutes

        val actual = validator.validate(mockBearerToken)

        assertThat(actual).isEqualTo(expected)
    }

    @Test
    fun `token with more less than 15 minutes, but more than 30 seconds is valid but should fetch new token`() {
        val expected = TokenValidator.Result.ShouldFetchNewToken
        every { mockBearerToken.remainingValidTime() } returns 31.seconds

        val actual = validator.validate(mockBearerToken)

        assertThat(actual).isEqualTo(expected)
    }

    @Test
    fun `token with less than 30 seconds left is expired`() {
        val expected = TokenValidator.Result.Expired
        every { mockBearerToken.remainingValidTime() } returns 29.seconds

        val actual = validator.validate(mockBearerToken)

        assertThat(actual).isEqualTo(expected)
    }


}
